jQuery.noConflict();
(function($, PLUGIN_ID) {

    'use strict';

    // utilities
    var util = window.util;
    var constant = util.constant;

    // params from user specifications
    var USER = kintone.getLoginUser();
    var LANG = USER.language;

    // multi-language settings
    var terms = {
        en: {
            configTitle: 'Settings',
            pluginActivation: 'Plug-in activation',
            pluginActive: 'Active',
            kintoneViewConfig: 'Kintone custom views with calendar',
            kintoneViews: 'Available views for calendar',
            kintoneFieldConfig: 'Kintone field settings for calendar',
            kintoneTitleField: 'Title',
            kintoneStartDatetimeField: 'Start datetime',
            kintoneEndDatetimeField: 'End datetime',
            kintoneAlldayField: 'All-day',
            kintoneScheduleColorField: 'Background color',
            pluginSubmit: '     Save   ',
            pluginCancel: '     Cancel   ',
            textCalendar: 'Calendar (plug-in)',
            textViews: 'The calendar will display in any "custom view” with <div id="calendar”/> in the html field.',
            textKintoneFields: 'Please create the following fields in your app form.',
            textNoViews: '(No custom views exist. Please create one <a id="create-customView">here</a>.)',
            textAllday: 'Please select the appropriate checkbox field.',
            textCreateViewTitle: 'Create custom view',
            textCreateViewText: 'Let\'s create a custom view for calender.',
            textUpdateViewSuccess: 'Custom view created',
            textUpdateViewError: 'Failed to get or update view',
            textApiError: 'Error occurred.',
            buttonCreateViewConfirm: 'Create',
            buttonCreateViewCancel: 'Cancel'
        },
        ja: {
            configTitle: '設定',
            pluginActivation: 'プラグインアクティベーション',
            pluginActive: '有効化',
            kintoneViewConfig: 'kintoneビュー設定',
            kintoneViews: 'カレンダー表示対象ビュー',
            kintoneFieldConfig: 'kintoneフィールド設定',
            kintoneTitleField: 'スケジュールタイトルフィールド',
            kintoneStartDatetimeField: 'スケジュール開始日時フィールド',
            kintoneEndDatetimeField: 'スケジュール終了日時フィールド',
            kintoneAlldayField: '終日チェックフィールド',
            kintoneScheduleColorField: 'スケジュール背景色フィールド',
            pluginSubmit: '     保存   ',
            pluginCancel: 'キャンセル',
            textCalendar: 'カレンダー (プラグイン)',
            textViews: '<div id="calendar"/> が設定されている全てのカスタマイズビューでカレンダー表示されます。',
            textKintoneFields: '次のフィールドを設定してください。',
            textNoViews: '(設定済みのカスタマイズビューが存在しません。<a id="create-customView">新しく作成する</a>。)',
            textAllday: '選択肢が1つのチェックボックスフィールドが選択可能です。（例：「終日」）',
            textCreateViewTitle: 'カスタマイズビュー作成',
            textCreateViewText: 'カレンダー表示用のカスタマイズビューを作成します。',
            textUpdateViewSuccess: 'カスタマイズビュー作成',
            textUpdateViewError: '一覧の取得もしくは更新に失敗しました。',
            textApiError: 'エラー発生',
            buttonCreateViewConfirm: '作成',
            buttonCreateViewCancel: 'キャンセル'
        }
    };
    var i18n = LANG in terms ? terms[LANG] : terms.en;

    // render view list
    var renderViewlist = function(items) {
        // '#viewlist-template' is defined in config.html
        var template = $.templates(document.querySelector('#viewlist-template'));
        $('#customCalender-view').empty().html(template(items));
    };

    // check if views for calendar are set
    var checkViews = function() {
        return kintone.api(kintone.api.url('/k/v1/preview/app/views', true), 'GET', {
            app: kintone.app.getId()
        }).then(function(r) {
            var views = r.views;
            var activeViews = [];
            Object.keys(views).forEach(function(view) {
                var id = $(views[view].html).prop('id');
                if ('#' + id === constant.FULL_CALENDAR.SELECTOR) {
                    activeViews.push({view: view});
                }
            });
            if (activeViews.length > 0) {
                renderViewlist({activeViews: activeViews});
            }
            r.activeViews = activeViews;
            return kintone.Promise.resolve(r);
        });
    };

    // update views or create custom view
    var updateViews = function() {
        return checkViews().then(function(r) {
            if (r.activeViews.length > 0) {
                return kintone.Promise.resolve({});
            }
            var updateParams = $.extend(true, {}, r);
            var viewName = i18n.textCalendar;
            updateParams.app = kintone.app.getId();
            updateParams.views[viewName] = {
                type: 'CUSTOM',
                name: viewName,
                html: '<div id="calendar"></div>',
                filterCond: '',
                pager: false,
                index: Object.keys(r.views).length
            };
            return kintone.api(kintone.api.url('/k/v1/preview/app/views', true), 'PUT', updateParams).then(function() {
                renderViewlist({activeViews: [{view: viewName}]});
            });
        });
    };

    // all-day flag value for multi-languages
    var getAlldayLabels = function(field) {
        var langs = constant.LANGUAGES;
        return kintone.Promise.all(langs.map(function(lang) {
            return kintone.api(kintone.api.url('/k/v1/preview/app/form/fields', true), 'GET', {
                app: kintone.app.getId(),
                lang: lang
            });
        })).then(function(resps) {
            var ret = {};
            resps.map(function(resp, i) {
                var label = '';
                var props = resp.properties;
                Object.keys(props).forEach(function(prop) {
                    if (prop === field) {
                        var options = Object.keys(props[prop].options);
                        if (options.length === 1) {
                            label = options[0];
                        }
                    }
                });
                return label;
            }).forEach(function(v, i) {
                ret[langs[i]] = v;
            });
            return ret;
        });
    };

    // append events
    var appendEvents = function appendEvents() {
        // available views for calendar are shown
        checkViews().catch(function(e) {
            swal(i18n.textUpdateViewError, e.message, 'error');
        });

        // create a new custom view
        $('#create-customView').click(function() {
            swal({
                title: i18n.textCreateViewTitle,
                text: i18n.textCreateViewText,
                type: 'warning',
                showCancelButton: true,
                confirmButtonColor: '#DD6B55',
                confirmButtonText: i18n.buttonCreateViewConfirm,
                cancelButtonText: i18n.buttonCreateViewCancel,
                closeOnConfirm: false
            }, function() {
                updateViews().then(function() {
                    swal(i18n.textUpdateViewSuccess, '', 'success');
                }).catch(function(e) {
                    swal(i18n.textUpdateViewError, e.message, 'error');
                });
            });
        });

        // save plug-in settings
        $('#submit').click(function() {
            var config = {};
            config.activation = $('#activation').prop('checked') ? 'active' : 'deactive';
            config.titleField = $('#title-field').val();
            config.startDatetimeField = $('#startDatetime-field').val();
            config.endDatetimeField = $('#endDatetime-field').val();
            config.alldayCheckField = $('#alldayCheck-field').val();
            config.bgcolorField = $('#bgcolor-field').val();
            getAlldayLabels(config.alldayCheckField).then(function(labels) {
                config.alldayCheckLabelJa = labels.ja;
                config.alldayCheckLabelEn = labels.en;
                config.alldayCheckLabelZh = labels.zh;
                kintone.plugin.app.setConfig(config);
            }).catch(function() {
                swal(i18n.textApiError, '', 'error');
            });
        });

        // cancel plug-in settings
        $('#cancel').click(function() {
            history.back();
        });
    };

    // create HTML (call in renderHtml)
    var createHtml = function(fields) {
        // template & items settings
        // '#plugin-template' is defined in config.html
        var template = $.templates(document.querySelector('#plugin-template'));
        var templateItems = {
            configTitle: i18n.configTitle,
            // section1 activate plug-in
            pluginActivation: {
                pluginActivation: i18n.pluginActivation,
                pluginActive: i18n.pluginActive
            },
            // section2 kintone view settings
            kintoneViewConfig: {
                kintoneViewConfig: i18n.kintoneViewConfig,
                kintoneViews: i18n.kintoneViews,
                textViews: i18n.textViews,
                textNoViews: i18n.textNoViews
            },
            // section3 kintone fields settings
            kintoneFieldConfig: i18n.kintoneFieldConfig,
            textKintoneFields: i18n.textKintoneFields,
            kintoneFields: [{
                title: i18n.kintoneTitleField,
                require: '*',
                row: '',
                id: 'title-field',
                fields: fields['title-field']
            }, {
                title: i18n.kintoneStartDatetimeField,
                require: '*',
                row: '',
                id: 'startDatetime-field',
                fields: fields['startDatetime-field']
            }, {
                title: i18n.kintoneEndDatetimeField,
                require: '*',
                row: '',
                id: 'endDatetime-field',
                fields: fields['endDatetime-field']
            }, {
                title: i18n.kintoneAlldayField,
                require: '*',
                row: i18n.textAllday,
                id: 'alldayCheck-field',
                fields: fields['alldayCheck-field']
            }, {
                title: i18n.kintoneScheduleColorField,
                require: '',
                row: '',
                id: 'bgcolor-field',
                fields: fields['bgcolor-field']
            }],
            // section4 buttons
            pluginSubmit: i18n.pluginSubmit,
            pluginCancel: i18n.pluginCancel
        };
        // render HTML
        $('#plugin-container').html(template(templateItems));
    };

    // render HTML
    var renderHtml = function() {
        kintone.api(kintone.api.url('/k/v1/preview/app/form/fields', true), 'GET', {
            'app': kintone.app.getId()
        }, function(resp) {
            var fields = {
                'title-field': [],
                'startDatetime-field': [],
                'endDatetime-field': [],
                'bgcolor-field': [],
                'alldayCheck-field': []
            };
            for (var key in resp.properties) {
                var field = resp.properties[key];
                var item = {
                    label: field.label || field.code,
                    code: field.code,
                    type: field.type
                };
                switch (field.type) {
                    case 'SINGLE_LINE_TEXT':
                        fields['title-field'].push(item);
                        fields['bgcolor-field'].push(item);
                        break;
                    case 'DATETIME':
                        fields['startDatetime-field'].push(item);
                        fields['endDatetime-field'].push(item);
                        break;
                    case 'DROP_DOWN':
                        fields['bgcolor-field'].push(item);
                        break;
                    case 'CHECK_BOX':
                        //var firstLabel = '';
                        var options = Object.keys(field.options);
                        if (options.length === 1) {
                            //firstLabel = options[0];
                            //item.firstLabel = firstLabel;
                            fields['alldayCheck-field'].push(item);
                        }
                        break;
                    default:
                        break;
                }
            }
            Object.keys(fields).forEach(function(f) {
                fields[f].sort(function(a, b) {
                    var aa = a.label || a.code;
                    var bb = b.label || b.code;
                    aa = aa.toUpperCase();
                    bb = bb.toUpperCase();
                    if (aa < bb) {
                        return -1;
                    } else if (aa > bb) {
                        return 1;
                    }
                    return 0;
                });
            });
            createHtml(fields);
            // set existing values
            var config = kintone.plugin.app.getConfig(PLUGIN_ID);
            if (config) {
                $('#activation').prop('checked', config.activation === 'active');
                $('#title-field').val(config.titleField);
                $('#startDatetime-field').val(config.startDatetimeField);
                $('#endDatetime-field').val(config.endDatetimeField);
                $('#alldayCheck-field').val(config.alldayCheckField);
                $('#bgcolor-field').val(config.bgcolorField);
            }
            // append events
            appendEvents();
        });
    };

    // initiated
    $(document).ready(function() {
        renderHtml();
    });
})(jQuery, kintone.$PLUGIN_ID);
