jQuery.noConflict();
(function($) {
    'use strict';

    var util = {
        constant: {
            TIMEOUT: 10000,
            INTERVAL: 10,
            MAX_READ_COUNT: 500,
            MAX_WRITE_COUNT: 100,
            FULL_CALENDAR: {
                SELECTOR: '#calendar',
                DEFAULT_EVENT_COLOR: '#69aeb3',
                DEFAULT_EVENT_TEXT_COLOR: 'white'
            },
            LANGUAGES: ['ja', 'en', 'zh'],
            // timezones cybozu.com defines
            // https://help.cybozu.com/ja/general/admin/timezone.html
            TIMEZONES: [
                'Etc/GMT+12',
                'Etc/GMT+11',
                'Pacific/Honolulu',
                'America/Anchorage',
                'America/Santa_Isabel',
                'America/Los_Angeles',
                'America/Chihuahua',
                'America/Phoenix',
                'America/Denver',
                'America/Guatemala',
                'America/Chicago',
                'America/Regina',
                'America/Mexico_City',
                'America/Bogota',
                'America/Indiana/Indianapolis',
                'America/New_York',
                'America/Caracas',
                'America/Halifax',
                'America/Asuncion',
                'America/La_Paz',
                'America/Cuiaba',
                'America/Santiago',
                'America/St_Johns',
                'America/Sao_Paulo',
                'America/Godthab',
                'America/Cayenne',
                'America/Argentina/Buenos_Aires',
                'America/Montevideo',
                'Etc/GMT+2',
                'Atlantic/Cape_Verde',
                'Atlantic/Azores',
                'Africa/Casablanca',
                'Atlantic/Reykjavik',
                'Europe/London',
                'Etc/GMT',
                'Europe/Berlin',
                'Europe/Paris',
                'Africa/Lagos',
                'Europe/Budapest',
                'Europe/Warsaw',
                'Africa/Windhoek',
                'Europe/Istanbul',
                'Europe/Kiev',
                'Africa/Cairo',
                'Asia/Damascus',
                'Asia/Amman',
                'Africa/Johannesburg',
                'Asia/Jerusalem',
                'Asia/Beirut',
                'Asia/Baghdad',
                'Europe/Minsk',
                'Asia/Riyadh',
                'Africa/Nairobi',
                'Asia/Tehran',
                'Europe/Moscow',
                'Asia/Tbilisi',
                'Asia/Yerevan',
                'Asia/Dubai',
                'Asia/Baku',
                'Indian/Mauritius',
                'Asia/Kabul',
                'Asia/Tashkent',
                'Asia/Karachi',
                'Asia/Colombo',
                'Asia/Kolkata',
                'Asia/Kathmandu',
                'Asia/Almaty',
                'Asia/Dhaka',
                'Asia/Yekaterinburg',
                'Asia/Rangoon',
                'Asia/Bangkok',
                'Asia/Novosibirsk',
                'Asia/Krasnoyarsk',
                'Asia/Ulaanbaatar',
                'Asia/Shanghai',
                'Australia/Perth',
                'Asia/Singapore',
                'Asia/Taipei',
                'Asia/Irkutsk',
                'Asia/Seoul',
                'Asia/Tokyo',
                'Australia/Darwin',
                'Australia/Adelaide',
                'Australia/Hobart',
                'Asia/Yakutsk',
                'Australia/Brisbane',
                'Pacific/Port_Moresby',
                'Australia/Sydney',
                'Asia/Vladivostok',
                'Pacific/Guadalcanal',
                'Etc/GMT-12',
                'Pacific/Fiji',
                'Asia/Magadan',
                'Pacific/Auckland',
                'Pacific/Tongatapu',
                'Pacific/Apia'
            ]
        },
        escapeHtml: function(string) {
            if (typeof string !== 'string') {
                return string;
            }
            return string.replace(/[&'`"<>]/g, function(match) {
                return {
                    '&': '&amp;',
                    "'": '&#x27;',
                    '`': '&#x60;',
                    '"': '&quot;',
                    '<': '&lt;',
                    '>': '&gt;'
                }[match];
            });
        },
        ui: {
            // スピナーを動作させる関数
            showSpinner: function() {
                // 要素作成等初期化処理
                if ($('.kintone-spinner').length === 0) {
                    // スピナー設置用要素と背景要素の作成
                    var spin_div = $('<div id ="kintone-spin" class="kintone-spinner"></div>');
                    var spin_bg_div = $('<div id ="kintone-spin-bg" class="kintone-spinner"></div>');
                    // スピナー用要素をbodyにappend
                    $(document.body).append(spin_div, spin_bg_div);
                    // スピナー動作に伴うスタイル設定
                    $(spin_div).css({
                        'position': 'fixed',
                        'top': '50%',
                        'left': '50%',
                        'z-index': '50010',
                        'background-color': '#fff',
                        'padding': '26px',
                        '-moz-border-radius': '4px',
                        '-webkit-border-radius': '4px',
                        'border-radius': '4px'
                    });
                    $(spin_bg_div).css({
                        'position': 'fixed',
                        'top': '0px',
                        'z-index': '50000',
                        'width': '150%',
                        'height': '150%',
                        'background-color': '#000',
                        'opacity': '0.5',
                        'filter': 'alpha(opacity=50)',
                        '-ms-filter': "alpha(opacity=50)"
                    });
                    // スピナーに対するオプション設定
                    var opts = {
                        'color': '#000'
                    };
                    // スピナーを作動
                    new Spinner(opts).spin(document.getElementById('kintone-spin'));
                }
                // スピナー始動（表示）
                $('.kintone-spinner').show();
            },
            // スピナーを停止させる関数
            hideSpinner: function() {
                // スピナー停止（非表示）
                $('.kintone-spinner').hide();
            },
            // kintoneスタイルのselect
            createStyledSelect: function($select) {
                return $('<div class="kintoneplugin-select-outer"></div>')
                    .append($('<div class="kintoneplugin-select"></div>')
                        .append($select));
            },
            // kintoneスタイルのradio
            createStyledRadio: function($radio, $label) {
                return $('<span class="kintoneplugin-input-radio-item">').append($radio).append($label);
            },
            // kintoneスタイルのradioの括り
            createStyledRadios: function($div) {
                return $div.addClass("kintoneplugin-input-radio");
            },
            // kintoneスタイルのcheckbox
            createStyledCheckbox: function($check, $label) {
                return $('<span class="kintoneplugin-input-checkbox-item kintoneplugin-input-checkbox-item-inline">')
                    .append($check)
                    .append($label);
            },
            // kintoneスタイルのcheckboxの括り
            createStyledCheckboxes: function($div) {
                return $div.addClass("kintoneplugin-input-checkbox");
            },
            // kintoneスタイルのinput/text
            createStyledText: function($input) {
                $input.prop('type', 'text').addClass('kintoneplugin-input-text');
                return $('<div class="kintoneplugin-input-outer">').append($input);
            },
            // kintoneスタイルのbutton
            createStyledButton: function($button, mode) {
                var classname;
                switch (mode) {
                    case 'ok':
                        classname = 'kintoneplugin-button-dialog-ok';
                        break;
                    case 'cancel':
                        classname = 'kintoneplugin-button-dialog-cancel';
                        break;
                    case 'disabled':
                        classname = 'kintoneplugin-button-disabled';
                        break;
                    default:
                        classname = 'kintoneplugin-button-normal';
                        break;
                }
                $button.addClass(classname);
                return $button;
            },
            // 一覧のヘッダへの挿入
            appendToIndexHeader: function() {
                var $menu;
                if (!$('.yamaroo-index-header-menu')[0]) {
                    $menu = $('<div>').prop({
                        class: 'yamaroo-index-header-menu yamaroo'
                    });
                    $(kintone.app.getHeaderMenuSpaceElement()).append($menu);
                } else {
                    $menu = $('.yamaroo-index-header-menu');
                }
                $.each(arguments, function(i, v) {
                    $menu.append(v);
                });
                $menu.children().css({
                    'vertical-align': 'top'
                });
            },
            // 詳細のヘッダへの挿入
            appendToDetailHeader: function() {
                var $menu;
                if (!$('.yamaroo-detail-header-menu')[0]) {
                    $menu = $('<div>').prop({
                        class: 'yamaroo-detail-header-menu yamaroo'
                    });
                    $(kintone.app.record.getHeaderMenuSpaceElement()).append($menu);
                } else {
                    $menu = $('.yamaroo-detail-header-menu');
                }
                $.each(arguments, function(i, v) {
                    $menu.append(v);
                });
                $menu.css({
                    'padding-left': 16,
                    'padding-top': 16
                });
                $menu.children().css({
                    'vertical-align': 'top'
                });
            }
        }
    };
    window.util = util;
})(jQuery);
